import unittest
from wkregister.decorator import record  # Import your decorator
from wkregister.models import Records
import asyncio


@record()
def add(a: float, b: float):

    record = Records(
        org="nmi",
        key="commonsense",
        userId="2",
        actionType="update",
        status="success",
        errorMessage=None,
        service="pay-service",
        payload={"hola": "mundo"},
    )
    return {"result": a + b, "record": record}


async def saveRecord():
    # Await the asynchronous method
    record = Records(
        org="commonsense",
        key="unittest",
        userId="2",
        actionType="update",
        status="success",
        errorMessage=None,
        service="pay-service",
        payload={"hola": "mundo"},
    )
    await record.save()


class TestLogs(unittest.TestCase):
    @unittest.skip
    def test_log(self):
        # Await the asynchronous method
        asyncio.run(add(12, 12))

        # Assert your test condition
        self.assertTrue(True)

    def test_register(self):
        # Await the asynchronous method
        record = Records(
            org="commonsense",
            key="unittest",
            userId="2",
            actionType="update",
            status="success",
            errorMessage=None,
            service="pay-service",
            payload={"hola": "mundo"},
        )

        asyncio.run(saveRecord())

        # Assert your test condition
        self.assertTrue(True)



# from wkregister.models import Records
from wkregister.producer import record2Kafka
from wkregister.util import kafkaParams
from typing import Callable, Any, Dict
from functools import wraps
import inspect
import asyncio
import json
import time


# Decorator for logging
def record():
    def decorator_log(func: Callable):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            input = func(*args, **kwargs)

            if inspect.iscoroutine(input):
                result = await input
            else:
                result = input

            org, key, log = kafkaParams(result)

            start_time = time.time()  # Capture the start time

            # Log after function execution, you can adjust what you log as needed
            asyncio.create_task(record2Kafka(org, key, log.dict()))

            end_time = time.time()  # Capture the end time

            elapsed_time = end_time - start_time  # Calculate elapsed time
            # print(f"The function took {elapsed_time} seconds to complete.")

            output = {}
            for key, value in result.items():
                if not key == "record" or not key == "log":
                    output[key] = value

            if "record" in output:
                del output["record"]
            if "log" in output:
                del output["log"]

            return output

        return wrapper

    return decorator_log



from wkregister.decorator import record, Records
import asyncio


@record()
def add(a: float, b: float):

    record = Records(
        org="nmi",
        key="testOrg4",
        userId="2",
        actionType="update",
        status="success",
        errorMessage=None,
        service="pay-service",
        payload={"hola": "mundo"},
    )
    return {"result": a + b, "record": record}


asyncio.run(add(1, 2))



from wkregister.producer import record2Kafka
from dataclasses import dataclass, field, asdict
from typing import Dict
import uuid
from datetime import datetime
import asyncio


@dataclass
class Records:
    org: str = ""
    key: str = ""
    userId: str = ""
    actionType: str = ""
    status: str = ""
    errorMessage: str = ""
    service: str = ""
    id: str = field(default_factory=lambda: str(uuid.uuid4()))
    timestamp: str = field(default_factory=lambda: str(datetime.today()))
    payload: Dict = field(default_factory=dict)

    def dict(self):
        return {k: str(v) for k, v in asdict(self).items()}

    def kafkaParams(self):

        # Check if 'org' or 'key' was not set and inform the user
        if not self.org:
            print("Org was not set")
        if not self.key:
            print("Key was not set")

        # Return the extracted values
        return self.org, self.key

    def createRecord(self):
        return {
            "org": self.org,
            "key": self.key,
            "userId": self.userId,
            "actionType": self.userId,
            "status": self.status,
            "errorMessage": self.errorMessage,
            "service": self.service,
            "id": self.id,
            "timestamp": self.timestamp,
            "payload": self.payload,
        }

    def validate(self):
        # Check if 'org' or 'key' was not set and inform the user
        if not self.org:
            print("Org was not set")
        if not self.key:
            print("Key was not set")

    async def save(self):

        self.validate()
        # Log after function execution, you can adjust what you log as needed
        asyncio.create_task(record2Kafka(self.org, self.key, self.createRecord()))
